;=========================================================================
;Bernd Kernbaum
;DK3WX
;=========================================================================
;
;PIC 4 MHz Quarztakt
;Funktion:
;
;=========================================================================
;Konstanten fuer die PIC-Konfiguration
;=========================================================================
#define ptype	873			; 873 oder 876

;********************************
	if ptype == 876
;********************************

	list      p=16f876             ; list directive to define processor
	#include <p16f876.inc>         ; processor specific variable definitions
	__config   _cp_off & _lvp_off & _wdt_off & _pwrte_on & _hs_osc

;********************************
	else
;********************************

	list      p=16f873             ; list directive to define processor
	#include <p16f873.inc>         ; processor specific variable definitions
	__config   _cp_off & _lve_off & _wdt_off & _pwrte_on & _hs_osc

;********************************
	endif
;********************************


;==========================================================
; Variablen:
;==========================================================
        org     020h

rette_w:	res     1	; W retten in der ISR  
rette_f:	res     1     	; f retten in der ISR
rette_p:	res     1     	; pclath retten in der ISR


bcounter:		res	1	;Bitzaehler 	
sr:		res	1	;Empfangsschieberegister
temp:		res	1	;Zaehler fuer Warteschleifen
temp1:		res	1	;Zaehler fuer Warteschleifen
werth:		res	1	;Messwert des AD Wandlers
wertl:		res	1	;Messwert des AD Wandlers
	
f_b0:		res	1      	;Frequenzspeicher zur dualen Rechnung
f_b1:		res	1      	;"
f_b2:		res	1      	;"
f_b3:		res	1      	;"
f_b4:		res	1      	;Konfigurationsbyte DDS-IC 

sr0:		res	1      	;Schieberegister
sr1:		res	1      	;"
sr2:		res	1      	;"
sr3:		res	1      	;"
sr4:		res	1      	;"

;=========================================================================
;Namen der I/O Leitungen 
;=========================================================================

;port a
;#define e_mess_a0	porta,0h        ;Analogeingang log.
;#define e_mess_a1	porta,1h        ;Analogeingang lin.
;#define e_mess_a2	porta,2h        ;Analogeingang 
;#define e_mess_a3	porta,3h        ;Analogeingang 


;port b mode Taste
#define e_typesel   	portb,0h        ;DDS Type whlen
#define e_pictakt   	portb,1h        ;Taktfrequenz PIC 
#define e_baud   	portb,2h        ;Baudrate whlen
#define e_version0   	portb,3h        ;Software Version 
#define e_version1   	portb,4h        ;Software Version 



;port c

#define a_data  	portc,0h        ;Daten DDS
#define a_reset   	portc,1h        ;reset DDS
#define a_fqud	  	portc,2h        ;Latch DDS
#define a_takt	  	portc,3h        ;Takt DDS
#define a_rts   	portc,4h        ;RTS_OUTPUT
#define e_cts   	portc,5h	;CTS_INPUT
#define a_sio		portc,6h   	;Daten zum PC
#define e_sio   	portc,7h	;Datenempfang vom PC


;=========================================================================
;Konstanten fuer die PIC-Konfiguration
;=========================================================================
d_option_reg:       	equ   	00h
d_port_a:           	equ   	00h
d_port_b:           	equ   	00h
d_port_c:           	equ   	00h
d_tmr0:             	equ   	00h
						;0=Ausgang
						;1=Eingang
d_trisa:            	equ   	b'11111111'	;Funktion siehe Definitionen

d_trisb:            	equ   	b'11111111' 

d_trisc:            	equ   	b'10110000'

d_intcon:           	equ   	00h		;global interrupt disable
d_adcon1:           	equ   	00h    		;PortA als 5 analog. Eingaenge  
						; ..linksbuendig
d_t1con:		equ	b'00100001'	;Timer 1 intern VT8 go
d_pie1:			equ	b'00000000'	;Timer 1 enable
d_pie2:			equ	b'00000000'	


						;Fosz ist  4MHz
baud_9600		equ	0x19		;Konstante fuer 9600 Baud
baud_19200		equ	0x0c		;Konstante fuer 19200 Baud
						;Fosz ist  10MHz
baud_38400		equ	0x0f		;Konstante fuer 38400 Baud
baud_57600		equ	0x0a		;Konstante fuer 57600 Baud

bank0:              	equ   	00h
bank1:              	equ   	80h
bank2:              	equ   	100h
bank3:              	equ   	180h


;=========================================================================
; Hardware initialisieren
;=========================================================================
	org	0x0000
	goto	anf
	nop
	nop
	nop

anf:    
        clrf        status        ; Flags loeschen
            
        banksel bank1             ; Registerbank 80h..0AFh
        movlw       d_option_reg
        movwf       option_reg
	movlw	    d_pie1
        movwf       pie1
	movlw	    d_pie2
        movwf       pie2
        movlw       d_trisa
        movwf       trisa
        movlw       d_trisb
        movwf       trisb
        movlw       d_trisc
        movwf       trisc
        movlw       d_adcon1
        movwf       adcon1
        banksel bank0             ; Registerbank 0..2FH
          

        movlw       d_port_a
        movwf       porta
        movlw       d_port_b
        movwf       portb
        movlw       d_port_c
        movwf       portc
        movlw       d_intcon
        movwf       intcon
	movlw	    d_t1con
	movwf	    t1con
	clrf	pir1
	clrf	ccp1con
	clrf	ccp2con

;=========================================================================
;Hauptprogramm
;=========================================================================

haupt:	
	call	serialsetup
	movlw	28h		; 1 MHz Sinus bei 100 MHz Takt
	movwf	f_b0
	movlw	5ch
	movwf	f_b1
	movlw	8fh
	movwf	f_b2
	movlw	02h
	movwf	f_b3
	clrf	f_b4
	btfsc	e_typesel	; Jumper Type select abfragen  
	bsf	f_b4,0

	bsf	a_reset		; RESET 
	nop
	nop
	nop
	nop
	nop
	nop
	bcf	a_reset
	nop
	nop
	nop
	nop
	nop
	bsf	a_takt		; Umschalten auf serieller Eingabe
	nop			; D0,D1 high, D2 bis D6 low, D7 Dateneingang
	nop
	bcf	a_takt
	nop
	nop
	bsf	a_fqud
	nop
	nop
	bcf	a_fqud
	nop
	nop
	
	call	ausg        	;erste ausgabe an AD9850/1
	call	ausg	


hloop:	call	rxd		;Byte 0
	movwf	f_b0
	call	rxd		;Byte 1
	movwf	f_b1
	call	rxd		;Byte 2
	movwf	f_b2
	call	rxd		;Byte 3
	movwf	f_b3

	call	ausg		; Frequenz an den DDS Chip

	call	rxd		; warten auf den Messbeginn
	
	movwf	temp
	andlw	0f0h
	xorlw	50h
	btfsc	status,z
	goto	hloop0
	call	ad0mess8	; Messen 
	call	txd		; Messwert zum PC
	goto	hloop

hloop0:
	movf	temp,w
	andlw	07h
	addwf	pcl,f
	goto	hloop1		;50h AD0 8Bit
	goto	hloop2		;51h AD1 8Bit
	goto	hloop3		;52h AD2 8Bit
	goto	hloop4		;53h AD3 8Bit
	goto	hloop5		;54h AD0 10Bit
	goto	hloop6		;55h AD1 10Bit
	goto	hloop7		;56h AD2 10Bit
	goto	hloop8		;57h AD3 10Bit
	goto	hloop

hloop1:
	movlw	041h
	call	adxmess
	call	txd
	goto	hloop
hloop2:
	movlw	049h
	call	adxmess
	call	txd
	goto	hloop
hloop3:
	movlw	051h
	call	adxmess
	call	txd
	goto	hloop
hloop4:
	movlw	059h
	call	adxmess
	call	txd
	goto	hloop
hloop5:
	movlw	041h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop6:
	movlw	049h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop7:
	movlw	051h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop8:
	movlw	059h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop


;=========================================================================
;Initialisieren der UART Port RC4 bis RC7 
;=========================================================================

serialsetup:
	banksel	bank0		;
	bsf	a_rts           ;RTS aus
	banksel	bank1		;
	bcf	trisc,4         ;RTS Pin wird Ausgang

	btfss	e_pictakt	;mit welcher Taktfrequenz arbeitet der PIC?
	goto	takt4mhz

takt10mhz:
				;10 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_38400	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_57600	;Setze Baudrate
	goto	setbaudrate

takt4mhz:
				;4 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_9600	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_19200	;Setze Baudrate

setbaudrate:
	movwf	spbrg
	bsf	txsta,brgh	;baud rate high speed option
	bsf	txsta,txen	;enable transmission
	banksel	bank0		;
	bsf	rcsta,cren	;enable reception
	bsf	rcsta,spen	;enable serial port
	return


;=========================================================================
;serieller Empfang von 8 Bit an RC7 
;=========================================================================
rxd:
	bcf	a_rts           ;setze RTS fuer Datenempfang
	btfss	pir1,rcif	;test ob Daten 
	goto	$-1		;..wenn nein, dann warten
	movf	rcreg,w		;Daten ins W-Register
	return

;=========================================================================
;Senden von 8 Bit an RC6 
;=========================================================================
txd_cts:
	btfsc	e_cts	        ;teste CTS ob gesendet werden kann
	goto	$-1
txd:
	btfss	pir1,txif	;.. und ob der Sendepuffer leer ist
	goto	$-1
	movwf	txreg		;Sende Byte
	return


;=========================================================================
;A/D Wandler Wert abfragen
;=========================================================================
ad0mess8:
	movlw	0x41     	; anal. kanal RA0
adxmess:
	movwf	adcon0		; ... als Messeingang setzen
	call	zem		;kurz warten
	bsf	adcon0,2	;Start Messung
ad0me1:	btfsc	adcon0,2	;warten bis Messung beendet
	goto	ad0me1   	;Ergebnis in adresh und adresl
	banksel	bank1
	movf	adresl,w
	banksel	bank0
	movwf	wertl
	movf	adresh,w	;nur die oberen 8 Bit werden genutzt
	movwf	werth
	return

zem:	movlw	0x07   		; zeitschleife 26 us  
	movwf	temp
zem1:	decfsz	temp,f
	goto	zem1
	return


;=========================================================================
;Umladen ins Schieberegister
;=========================================================================
ausg:   bcf         a_fqud      ;umladen
        bcf         a_takt
        bcf         a_data
        movf        f_b0,w
        movwf       sr0
        movf        f_b1,w
        movwf       sr1
        movf        f_b2,w
        movwf       sr2
        movf        f_b3,w
        movwf       sr3
        movf        f_b4,w
        movwf       sr4

;=========================================================================
;Ausgabe an Frequenzregister des AD9850
;=========================================================================
ausg0:  movlw       28h		;40 dez Bit
        movwf       bcounter
rot:    bcf         a_takt
        bcf         status,0
        rrf         sr4,f
        rrf         sr3,f
        rrf         sr2,f
        rrf         sr1,f
        rrf         sr0,f
        btfss       status,0
        goto        rot0
        bsf         a_data
        goto        rotw
rot0:   bcf         a_data
rotw:   nop
        bsf         a_takt
        decfsz      bcounter,f
        goto        rot
        bcf         a_takt
        nop
        bsf         a_fqud
        nop
        nop
        nop
        nop
        bcf         a_fqud
        nop
        nop
        nop
        nop
        bcf         a_takt
        bcf         a_data
        return


        end
